`timescale 1ns / 1ps
//*****************************************************************************
// File Name            : ps2_ap.v
//-----------------------------------------------------------------------------
// Function             : MicroBlaze ps2  system 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2014/01/21
//*****************************************************************************

module ps2_ap(  Clk, Reset, UART_Rx, UART_Tx,led,dip_sw,
 ps2_clk,
 ps2_dat,              
 //monitor  
 mon_ps2_clk,
 mon_ps2_data,
 mon
    );

  input  Clk;      //input clock 100MHz
  input  Reset;    //reset Hi=reset
  input  UART_Rx;  //UART rx 115200bps
  output UART_Tx; //UART tx 115200bps
  output [3 : 0] led;   //led Hi=on
  input  [3 : 0] dip_sw; //input dip_sw
  inout ps2_clk;
  inout ps2_dat;
  output mon_ps2_clk;
  output mon_ps2_data;
  output         mon;
   
  // 
  parameter p_reg_addr_low =  32'hC0000000;
  parameter p_reg_addr_hi  =  32'hC0000007;
  parameter p_bram_addr_low = 32'hC0001000;
  parameter p_bram_addr_hi  = 32'hC00017FF;
  // 
  parameter p_wb_addr_low  =  32'hD0000000;
  parameter p_wb_addr_hi   =  32'hEFFFFFFF;
  parameter p_wb_ps2_low   =  p_wb_addr_low + 0;
  parameter p_wb_ps2_hi    =  p_wb_addr_low + 15;
  //IO Bus
  wire IO_Addr_Strobe;
  wire IO_Read_Strobe;
  wire IO_Write_Strobe;
  wire [31 : 0] IO_Read_Data;
  wire [31 : 0] IO_Read_Data_reg;
  wire [31 : 0] IO_Read_Data_wb;
  wire [31 : 0] IO_Address;
  wire [3 : 0]  IO_Byte_Enable;
  wire [31 : 0] IO_Write_Data;
  wire IO_Ready;
  wire IO_Ready_reg;
  wire IO_Ready_wb;
  wire iobus_reg_sel;
  wire iobus_wb_sel;   
  //timmer
  wire FIT1_Interrupt;
  wire FIT1_Toggle;    
  wire PIT1_Interrupt; 
  wire PIT1_Toggle; 
  //GPO   
  wire [31 : 0] GPO1;
  wire [31 : 0] GPI1;
  //GPI
  wire GPI1_Interrupt;
  //Interrupt controller
  wire INTC_Interrupt;
  wire INTC_IRQ;
  //iobus_reg
  wire [31:0] data_in;
  wire [3:0]  data_in_en;
  wire [31:0] data_out;
  wire [3:0]  data_out_en;
  //user_module
  wire [7:0] chk_data; 
  //block ram port-B
  wire  [3:0]web;
  wire  [8 : 0] addrb;
  wire  [31 : 0] dinb;
  wire  [31 : 0] doutb;
  //Wishbone bus microblaze -> IP
  wire         WB_CLK;         // clock 
  wire         WB_RST;         // reset (asynchronous active hi)
  wire         WB_CYC;         // cycle
  wire         WB_STB;         // strobe
  wire  [31:0] WB_ADR;         // address adr_i
  wire         WB_WE;          // write enable
  wire  [31:0] WB_DAT_I;          // data output
  wire  [31:0] WB_DAT_O;          // data input
  wire  [3:0]  WB_SEL;         // byte enable
  wire         WB_ACK;         // normal bus termination
  wire         WB_ERR;         // err bus termination
  wire         WB_RTY;         // rtry
  //ps2
  wire         cyc_i_ps2;         // cycle
  wire         stb_i_ps2;         // strobe
  wire         we_i_ps2;          // write enable
  wire  [15:0] adr_i_ps2;         // address adr_i
  wire  [7:0] dat_o_ps2;   // data input
  wire         ack_o_ps2;         // normal bus termination
  wire         wb_ps2_sel;
  wire         irq_o_ps2;
   
   
  reg [23:0] clk_cnt;
  reg ps2_clk_d1;
  reg ps2_clk_d2;
  reg ps2_clk_d3;
  reg mon_ps2_clk;
  reg ps2_data_d1;
  reg ps2_data_d2;
  reg ps2_data_d3;
  reg mon_ps2_data;
//mon
  reg         mon;

  //LED$B@)8f(B
  
  //assign led = {data_out[1:0],GPO1[1:0]};
  //assign led = {2'b00,o_pwm1,o_pwm0};
  assign led = {data_out[3:0]};
  //$B?.9f@\B3(B
  assign GPI1 = {28'h0000000,dip_sw};
  assign data_in = data_out;
  assign data_in_en = data_out_en; 
  assign chk_data = data_out[7:0];  
  assign INTC_Interrupt = 1'b0;  

  //$B<~JU2sO)A*Br?.9f(B
  assign iobus_reg_sel  = ((IO_Address>=p_reg_addr_low)&&(IO_Address<=p_reg_addr_hi))? 1'b1:1'b0; 
  assign iobus_wb_sel  =  ((IO_Address>=p_wb_addr_low)&&(IO_Address<=p_wb_addr_hi))? 1'b1:1'b0;
  //$BFI=P$7%G!<%?$NA*Br(B       
  assign IO_Read_Data = (iobus_reg_sel==1'b1)?  IO_Read_Data_reg:
                        (iobus_wb_sel==1'b1)?   IO_Read_Data_wb:
                                                32'h00000000;
  //Ready$B?.9f$NA*Br(B  
  assign IO_Ready =    (iobus_reg_sel==1'b1)?  IO_Ready_reg:
                       (iobus_wb_sel==1'b1)?   IO_Ready_wb:
                                               (IO_Write_Strobe||IO_Read_Strobe);

//Microblaze MCS$B$N%$%s%9%?%s%9(B
mb_mcs mcs_0 (
  .Clk(Clk), // input Clk
  .Reset(Reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready), // input IO_Ready
  .UART_Rx(UART_Rx), // input UART_Rx
  .UART_Tx(UART_Tx), // output UART_Tx
  .FIT1_Interrupt(FIT1_Interrupt), // output FIT1_Interrupt
  .FIT1_Toggle(FIT1_Toggle), // output FIT1_Toggle
  .PIT1_Interrupt(PIT1_Interrupt), // output PIT1_Interrupt
  .PIT1_Toggle(PIT1_Toggle), // output PIT1_Toggle
  .GPO1(GPO1), // output [31 : 0] GPO1
  .GPI1(GPI1), // input [31 : 0] GPI1
  .GPI1_Interrupt(GPI1_Interrupt), // output GPI1_Interrupt
  .INTC_Interrupt(INTC_Interrupt), // input [0 : 0] INTC_Interrupt
  .INTC_IRQ(INTC_IRQ) // output INTC_IRQ
); 
 
//$B@_Dj%l%8%9%?$N%$%s%9%?%s%9(B
iobus_reg
#(p_reg_addr_low, p_reg_addr_hi)
iobus_reg_0 (
  .Clk(Clk), // input Clk
  .Reset(Reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_reg), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_reg), // input IO_Ready
  //input 
  .data_in(data_in), // input data
  .data_in_en(data_in_en), // input data enable
  //output 
  .data_out(data_out), // output data
  .data_out_en(data_out_en) // output data enable
);

iobus2wb 
#(p_wb_addr_low, p_wb_addr_hi)
iobus2wb(
  .Clk(Clk), // input Clk
  .Reset(Reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_wb), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_wb), // input IO_Ready
  //wishbone MicorBlaze -> IP
  .CLK_O(WB_CLK),         // clock
  .RST_O(WB_RST),         // reset 
  .CYC_O(WB_CYC),         // cycle
  .STB_O(WB_STB),         // strobe
  .ADR_O(WB_ADR),         // address adr_i
  .SEL_O(WB_SEL),         // byte_sel
  .WE_O( WB_WE),          // write enable
  .DAT_O(WB_DAT_I),       // data output
  .DAT_I(WB_DAT_O),       // data input
  .ACK_I(WB_ACK),         // normal bus termination
  .ERR_I(WB_ERR),         // error bus termination
  .RTY_I(WB_RTY)          // retry
                  );
   
   
//opencores ps2core
 ps2_wb ps2_wb_0(
  .wb_clk_i(WB_CLK),       // clock
  .wb_rst_i(WB_RST),       // reset 
  .wb_stb_i(stb_i_ps2),    // strobe
  .wb_adr_i(adr_i_ps2[0]), // address adr_i
  .wb_we_i (we_i_ps2),      // write enable
  .wb_dat_i(WB_DAT_I[7:0]),// data output
  .wb_dat_o(dat_o_ps2),    // data input
  .wb_ack_o(ack_o_ps2),    // normal bus termination
  .irq_o   (irq_o_ps2),
  .ps2_clk (ps2_clk),
  .ps2_dat (ps2_dat)); 
   
//WihsBoneBus 
   assign wb_ps2_sel = ((WB_ADR>=p_wb_ps2_low)&&(WB_ADR<=p_wb_ps2_hi))? 1'b1:1'b0;
   assign cyc_i_ps2  =(wb_ps2_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_ps2  =(wb_ps2_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_ps2  ={12'h000,WB_ADR[3:0]};
   assign we_i_ps2   =(wb_ps2_sel==1'b1)?WB_WE:1'b0;   
   assign WB_DAT_O   =(wb_ps2_sel==1'b1)?{dat_o_ps2,dat_o_ps2,dat_o_ps2,dat_o_ps2}:
                                        32'h00000000;   
   assign WB_ACK = (wb_ps2_sel==1'b1)?ack_o_ps2:WB_STB;
   assign WB_ERR = 1'b0;
   assign WB_RTY = 1'b0;
   
always @ (posedge Clk )
  if (Reset==1'b1)
     clk_cnt <= 24'd0;
  else
     clk_cnt <= clk_cnt + 24'd1;
  
//mon
   
always @ (posedge Clk )
    begin
      ps2_clk_d1 <=  ps2_clk;
      ps2_clk_d2 <=  ps2_clk_d1;
      ps2_clk_d3 <=  ps2_clk_d2;
      mon_ps2_clk <=  ps2_clk_d3;
      ps2_data_d1 <=  ps2_dat;
      ps2_data_d2 <=  ps2_data_d1;
      ps2_data_d3 <=  ps2_data_d2;
      mon_ps2_data <=  ps2_data_d3;
    end // always @ (posedge clk )
   
always @ (posedge clk_cnt[6] )
  mon <= ^dat_o_ps2;
endmodule
